#version 330 core
#extension GL_ARB_explicit_attrib_location: enable

layout(location = 0) in vec3 vertexPositionIn;
layout(location = 1) in vec2 uvIn;
// rgb = block light, a=sun light level
layout(location = 2) in vec4 rgbaLightIn;
// rgb = ao, biome tint, block side shadings
layout(location = 3) in vec4 rgbaBlockIn;

// Bits 0-7: Glow level
// Bits 8-10: Z-Offset
// Bit 11: Wind waving yes/no
// Bit 12: Water waving yes/no
layout(location = 4) in int renderFlags;
layout(location = 5) in vec2 flowVector;

// Bit 0: Should wave yes/no
// Bit 1: Should texture fade yes/no
// Bits 8-15: x-Distance to upper left corner, where 255 = size of the block texture
// Bits 16-24: y-Distance to upper left corner, where 255 = size of the block texture
// Bit 25: Lava yes/no
layout(location = 6) in int waterFlagsIn;

uniform float waterStillCounter;

uniform vec4 rgbaFogIn;
uniform vec3 rgbaAmbientIn;
uniform float fogDensityIn;
uniform float fogMinIn;
uniform vec3 origin;
uniform mat4 projectionMatrix;
uniform mat4 modelViewMatrix;
uniform float blockTextureSize;

uniform vec3 playerViewVec;

out vec2 flowVectorf;
out vec4 rgba;
out vec4 rgbaFog;
out float fogAmount;
out vec2 uv;
out vec2 uvSize;
out float waterStillCounterOff;
out vec3 fragWorldPos;
out vec3 fragNormal;
flat out vec2 uvBase;
flat out int waterFlags;

#include shadowcoords.vsh
#include fogandlight.vsh
#include vertexwarp.vsh


void main(void)
{
	vec4 worldPos = vec4(vertexPositionIn + origin, 1.0);
	
	worldPos = applyVertexWarping(renderFlags | (waterFlagsIn & 1) << 12, worldPos);
	
	float x = mod(waterStillCounter + length(worldPos.xz + playerpos.xz)/3, 2);

	waterStillCounterOff = smoothstep(0, 1, abs(x - 1));
	if ((waterFlagsIn & 2) == 0) {
		waterStillCounterOff = 1;
	}
	
	vec4 cameraPos = modelViewMatrix * worldPos;
	
	
	gl_Position = projectionMatrix * cameraPos;
	
	fragWorldPos = worldPos.xyz + playerpos;
	
	
	rgba = applyLight(rgbaAmbientIn, rgbaLightIn, rgbaBlockIn, renderFlags, cameraPos);
	rgbaFog = rgbaFogIn;
	fogAmount = getFogLevel(worldPos, fogMinIn, fogDensityIn);
	uv = uvIn;
	
	float w= ((waterFlagsIn >> 8) & 0xff) / 255.0 * blockTextureSize;
	float h = ((waterFlagsIn >> 16) & 0xff) / 255.0 * blockTextureSize;
	uvSize = vec2(w, h);
	uvBase = uv - uvSize;
	
	flowVectorf = flowVector;
	
	waterFlags = waterFlagsIn;
	fragNormal = unpackNormal(renderFlags >> 15);
	vec3 normal = vec3(0,1,0);
	vec3 eye = normalize(vec3(worldPos.x, worldPos.y-3.5, worldPos.z));
	float bias = 0.1;
	float scale = 0.9;
	float power = 0.9;
	
	if ((renderFlags & 0xff) == 0) { // Don't apply to glowing liquids for now, looks weird on lava (makes it less glowy)
		float fresnel = bias + scale * pow(1 + dot(eye, normal), power);	
		rgba.a = min(fresnel, clamp(20 * (1.2 - length(worldPos) / viewDistance) - 5, 0, 1));
	}
	
	
	
	calcShadowMapCoords(modelViewMatrix, worldPos);
	
	// We pretend the decal is closer to the camera to enforce it always being drawn on top
	// Required e.g. when water is besides stairs or slabs
	gl_Position.w += 0.0008;
}