#version 330 core

uniform sampler2D terrainTex;

uniform float blockTextureSize;

in vec4 rgba; // Without biome tint
in vec4 rgba2; // With biome tint
in vec4 rgbaFog;
in float fogAmount;
in vec2 uv;
in vec2 uv2;
in float glowLevel;

in float normalShadeIntensity;
flat in int renderFlags;
flat in vec3 normal;


layout(location = 0) out vec4 outColor;
layout(location = 1) out vec4 outGlow;

#include fogandlight.fsh

void main() 
{
	vec4 brownSoilColor = texture(terrainTex, uv) * rgba;
	vec4 grassColor;
	
	if (rgba.a < 0.01) {
		// Bottom
		outColor = applyFog(brownSoilColor, fogAmount);
	} else {
		if (rgba2.a > 0.01) {
			// Top
			grassColor = texture(terrainTex, uv2) * rgba2;
		} else {
			// Side + Overlay
			grassColor = texture(terrainTex, uv2 + vec2(blockTextureSize, 0)) * vec4(rgba2.rgb, 1);
		}
		
		outColor = brownSoilColor * (1 - grassColor.a) + grassColor * grassColor.a;
	}
	

	if (((renderFlags >> 6) & 1) > 0) {
		outColor = applyFogAndShadowWithNormal(outColor, fogAmount, normal, normalShadeIntensity, 0.5);
	} else {
		outColor = applyFogAndShadow(outColor, fogAmount);
	}

	outColor.a = rgbaFog.a;
    outGlow = vec4(glowLevel, 0, 0, outColor.a);
}