#version 330 core
#extension GL_ARB_explicit_attrib_location: enable

layout(location = 0) in vec3 vertexPositionIn;
layout(location = 1) in vec2 uvIn;
layout(location = 2) in vec4 colorIn;

// Bits 0-7: Glow level
// Bits 8-10: Z-Offset
// Bit 11: Wind waving yes/no
// Bit 12: Water waving yes/no
// Bit 13: low contrast mode
// Bit 14-26: x/y/z normals, 12 bits total. Each axis with 1 sign bit and 3 value bits
layout(location = 3) in int renderFlagsIn;


uniform vec4 rgbaIn;
uniform int extraGlow;
uniform mat4 projectionMatrix;
uniform mat4 modelViewMatrix;
uniform mat4 modelMatrix;
uniform int applyModelMat;
uniform int applyColor;

out vec2 uv;
out vec2 uvOverlay;
out vec4 color;

flat out int renderFlags;
flat out vec3 normal;
out float normalShadeIntensity;

#include fogandlight.vsh

void main(void)
{
	uv = uvIn;
	
	float glowLevel = extraGlow / 64.0;
	
	color = rgbaIn * (1 + glowLevel);
	if (applyColor == 1) color *= colorIn;
	
	
	gl_Position = projectionMatrix * modelViewMatrix * vec4(vertexPositionIn, 1.0);
	
	// Lower 8 bit is glow level
	renderFlags = renderFlagsIn; 
	
	normal = unpackNormal(renderFlags >> 15);
	if (applyModelMat > 0) {
		normal = (modelMatrix * vec4(normal, 0)).xyz;
		normal = normalize(normal);
	}
	
	normalShadeIntensity = 1;
}