#version 330 core
#extension GL_ARB_explicit_attrib_location: enable

layout(location = 0) in vec3 vertexPositionIn;
layout(location = 1) in vec2 uvIn;
layout(location = 2) in vec4 modelColor;
layout(location = 3) in int flags;

uniform vec3 rgbaAmbientIn;
uniform vec4 rgbaLightIn;
uniform int extraGlow;
uniform mat4 projectionMatrix;
uniform mat4 modelViewMatrix;

out vec2 uv;
out vec4 color;

flat out int renderFlags;
flat out vec3 normal;
out float normalShadeIntensity;


#include shadowcoords.vsh
#include fogandlight.vsh

void main(void)
{
	vec4 cameraPos = modelViewMatrix * vec4(vertexPositionIn, 1.0);
	
	renderFlags = extraGlow + flags;
	
	uv = uvIn;
	color = applyLight(
		rgbaAmbientIn, 
		rgbaLightIn, 
		modelColor, 
		renderFlags,
		cameraPos
	);
	
	gl_Position = projectionMatrix * cameraPos;
	
	normal = unpackNormal(renderFlags >> 15);
	normalShadeIntensity = 1;
}