#version 330 core
#extension GL_ARB_explicit_attrib_location: enable

layout(location = 0) in vec3 vertexPosition;  // Per vertex
layout(location = 1) in vec2 uvIn;				// Per vertex
layout(location = 2) in vec4 rgbaBlockIn;		// Per vertex (rgb = block light, a=sun light level)
layout(location = 3) in int renderFlagsIn; 		// Per vertex

layout(location = 4) in vec4 rgbaLightIn;		// Per instance
layout(location = 5) in mat4 transform;	 	// Per instance

uniform vec4 rgbaFogIn;
uniform vec3 rgbaAmbientIn;	
uniform float fogMinIn;
uniform float fogDensityIn;
uniform mat4 projectionMatrix;
uniform mat4 modelViewMatrix;

out vec4 color;
out vec2 uv;
out vec4 rgbaFog;
out float fogAmount;
out vec3 normal;
flat out int renderFlags;

#include shadowcoords.vsh
#include fogandlight.vsh

void main()
{
	vec4 worldPos = transform * vec4(vertexPosition, 1.0);
	vec4 cameraPos = modelViewMatrix * worldPos;
	uv = uvIn;
	color = applyLight(rgbaAmbientIn, rgbaLightIn, rgbaBlockIn, renderFlagsIn, cameraPos);
	rgbaFog = rgbaFogIn;	
	
	gl_Position = projectionMatrix * cameraPos;
	
	fogAmount = getFogLevel(worldPos, fogMinIn, fogDensityIn);
	renderFlags = renderFlagsIn;
	
	normal = unpackNormal(renderFlagsIn >> 15);
	normal = normalize((transform * vec4(normal.x, normal.y, normal.z, 0)).xyz);
}